/**
 *   \file		lista.h
 *   \author	Tranchida Giulio, No. Matricola 241732\n
 *				Si dichiara che il contenuto di questo file e',
 * 				in ogni sua parte, opera originale dell'autore.\n\n
 *
 * This program is free software; you can redistribuite it and/or modify
 * it under the terms of the GNU/General Pubblic License as published the
 * Free software Foundation; either version 2 of the License, or
 * (at your opinion) any later version.
 * */
#ifndef LISTA_H_
#define LISTA_H_

#include "common.h"

/** Definisce il valore della lista vuota */
#define	EMPTY	0

/**
 * Definisce la dimensione massima della lista in
 * relazione al numero dei thread massimi ammessi
 * contemporaneamente in esecuzione */
#define FULL	THREAD_MAX

/** Tipo enumerativo booleano */
typedef enum {false, true} boolean;

/** Definizione dell'elemento della lista */
struct thread{
	channel_t sk_cl;		/**< socket di comunicazione associato al thread */					
	struct thread *next;	/**< puntatore al prossimo elemento della lista */
};

/** Definizione del puntatore agli elementi della lista */
struct lista{
	int num_threads;		/**< contiene il numero di elementi presenti nella lista */
	struct thread *top_elem;/**< puntatore al primo elemento della lista */
};

/** 
 * <H3>inizializza</H3> 
 * Funzione che inizializza lo stack\n
 * 
 * @param *stack
 *
 * */ 
void inizializza(struct lista *stack);

/**
 * <H3>ins_new_elem</H3>
 *  Inserisce un elemento nello stack
 * 
 * @param sk_cl
 * @param *stack
 * @returns
 * 	- 0 in caso di successo
 * 	- errno in caso di errore
 *  */
int ins_new_elem(channel_t sk_cl, struct lista *stack);

/**
 * <H3>estrazione</H3>
 *  Estrae l'elemento in cima allo stack 
 * 
 * @param *stack
 * @returns
 * 	- channel_t, l'identificatore del canale di comunicazione
 * 	- -1 se l alista e' vuota
 * */
channel_t estrazione(struct lista *stack);

/**
 * <H3>full</H3>
 * Controlla se la coda e' piena o meno.
 *  
 * @param *stack
 * @returns
 * In accordo con il tipo "boolean" restituisce
 * 	- true se piena
 * 	- false altrimenti
 * */
boolean full(const struct lista *stack);

/**
 * <H3>rimuovi</H3>
 * Rimuove dalla lista un elemento con chiave "sk_cl" e lo disalloca.
 *
 * \param s   puntatore alla lista
 * \param sk_cl cidentificatore del canale di comunicazione
 *
 * @returns
 *   0 se tutto e' andato bene
 *  -1 se l'elemento non e' presente
 *  
 * */
int rimuovi(channel_t sk_cl, struct lista *s);

/**
 * <H3>leggi_sk_cl</H3>
 *  Restituisce il l'id del socket dell'elemento in cima allo stack
 *  
 * @param *stack
 * @returns
 * 		channel_t, l'identificatore del canale di comunicazione
 * 		tra client e server
 * */
channel_t leggi_sk_cl(struct lista *stack);

#endif /*LISTA_H_*/
