/**
 * 	\file		comandi.c
 *	\author		Tranchida Giulio, No Matricola 241732\n
 *				Si dichiara che il contenuto di questo file e',
 * 				in ogni sua parte, opera originale dell'autore.\n\n
 *
 * This program is free software; you can redistribuite it and/or modify
 * it under the terms of the GNU/General Pubblic License as published the
 * Free software Foundation; either version 2 of the License, or
 * (at your opinion) any later version.
 * */
#include "Include/comandi.h"

/* Funzioni ad uso e consumo della sola funzione 'thread_servente' */

/**
 * Gestisce l'esecuzione del messaggio di creazione directory
 * inviato dal client, eseguendo la funzione "fat_mkdir" della
 * libreria sfat e controllando il valore di ritorno della funzione.
 * A seconda del valore di ritorno della funzione "fat_mkdir",
 * verra', in caso di errore, invocata la funzione "sfaterror"
 * della libreria sfat.\n
 * Altrimenti la funzione creera' un messaggio di type "MSG_OK", con
 * buffer nullo e lenght 0.\n
 * Un mutex posto sulla chiamata della funzione "fat_mkdir", assicura
 * che non si verifichino problemi di race condiction.
 * 
 * \param msg puntatore alla struttura messagge_t
 * 
 * \returns ret_val, ovvero il valore ritornato dalla "fat_mkdir".
 * 
 * */
static int msg_mkdir(message_t *msg);

/**
 * Gestisce l'esecuzione del messaggio di listing directory
 * inviato dal client, eseguendo la funzione "fat_ls" della
 * libreria sfat e controllando il valore di ritorno della funzione.
 * A seconda del valore di ritorno della funzione "fat_ls",
 * verra', in caso di errore, invocata la funzione "sfaterror"
 * della libreria sfat.\n
 * Altrimenti la funzione creera' un messaggio di type "MSG_OK".\n Il
 * buffer sara' una stringa che e' la lista delle subdirectory e dei
 * file contenuti all'interno di in una cartella di cui si e' richiesto
 * la list.\n Lenght sara' pari alla lunghezza di questa stringa.\n
 * Un mutex posto sulla chiamata della funzione "fat_ls", assicura
 * che non si verifichino problemi di race condiction.
 * 
 * \param msg puntatore alla struttura messagge_t
 * 
 * \returns ret_val, ovvero il valore ritornato dalla  "fat_ls".
 * 
 * */
static int msg_ls(message_t *msg);

/**
 * Gestisce l'esecuzione del messaggio di creazione file inviato
 * dal client, eseguendo la funzione "fat_open" della libreria
 * sfat e controllando il valore di ritorno della funzione.
 * A seconda del valore di ritorno della funzione "fat_open",
 * verra', in caso di errore, invocata la funzione "sfaterror"
 * della libreria sfat.\n
 * Altrimenti la funzione creera' un messaggio di type "MSG_OK",
 * con buffer nullo e lenght 0.\n
 * Un mutex posto sulla chiamata della funzione "fat_ls", assicura
 * che non si verifichino problemi di race condiction.
 * 
 * \param msg puntatore alla struttura messagge_t
 * 
 * \returns ret_val, ovvero il valore ritornato dalla  "fat_open".
 * 
 * */
static int msg_mkfile(message_t *msg);

/**
 * Gestisce l'esecuzione del messaggio di creazione file inviato
 * dal client, eseguendo la funzione "fat_read" della libreria
 * sfat e controllando il valore di ritorno della funzione.\n
 * La funzione si occupa anche di interpretare il buffer del
 * messaggio ricevuto dal server, scindendolo in tre parti:
 *  - l'indirizzo di partenza in cui si inizia a leggere dal file,
 *  - la path del file da leggere e 
 *  - la dimensione dei dati da leggere.
 * 
 * A seconda del valore di ritorno della funzione "fat_read",
 * verra', in caso di errore, invocata la funzione "sfaterror"
 * della libreria sfat.\n
 * Altrimenti la funzione creera' un messaggio di type "MSG_OK",
 * il buffer sara' una stringa contenente i dati letti dal file, 
 * mentre lenght sara' la lunghezza di questa stringa.\n
 * Un mutex posto sulla chiamata della funzione "fat_ls", assicura
 * che non si verifichino problemi di race condiction.
 * 
 * \param msg puntatore alla struttura messagge_t
 * 
 * \returns ret_val, ovvero il valore ritornato dalla  "fat_read".
 * 
 * */
static int msg_fread(message_t *msg);

/**
 * Gestisce l'esecuzione del messaggio di scrittura su file
 * inviato dal client, eseguendo la funzione "fat_write" della
 * libreria sfat e controllando il valore di ritorno della
 * funzione.\n
 * La funzione si occupa anche di interpretare il buffer del
 * messaggio ricevuto dal server, scindendolo in due pezzi,
 * la path del file in cui scrivere e "data", ovvero i dati 
 * che bisogna scrivere nel file stesso.\n
 * A seconda del valore di ritorno della funzione "fat_write",
 * verra', in caso di errore, invocata la funzione "sfaterror"
 * della libreria sfat.\n
 * Altrimenti la funzione creera' un messaggio di type "MSG_OK",
 * con buffer nullo e lenght 0.\n 
 * Un mutex posto sulla chiamata della funzione "fat_ls", assicura
 * che non si verifichino problemi di race condiction.
 * 
 * \param msg puntatore alla struttura messagge_t
 * 
 * \returns ret_val, ovvero il valore ritornato dalla  "fat_write".
 * 
 * */
static int msg_fwrite(message_t *msg);

void *thread_servernte(void * sk){
	extern struct lista my_socket;
	extern pthread_mutex_t lock;
	extern pthread_cond_t wait_me;
	extern int num_threads;
	sigset_t set;		/**< Maschera per bloccare tutti i segnali */
	channel_t sk_cl;
	message_t msg;
	int recive_val=0;

	sk_cl = *(int *)sk;
	msg.buffer = NULL;

	/* Blocco i segnali */
	if (sigfillset(&set) == -1){
		errore(__FILE__,__LINE__,"Server: error sigfillset",errno);
		return 0;
	}
	if (pthread_sigmask(SIG_SETMASK, &set, NULL) != 0) {
		errore(__FILE__,__LINE__,"Server: pthread_sigmask error 'sigsetmask'",errno);
		return 0;
	}

	while (recive_val!=SEOF){
		if ((recive_val = receiveMessage(sk_cl, &msg)) == -1) break;
		else if (recive_val != SEOF){	/* !SIGPIPE */
			switch(msg.type){
				case MSG_MKDIR:
					msg_mkdir(&msg);
				break;
				case MSG_LS:
					msg_ls(&msg);
				break;
				case MSG_MKFILE:
					msg_mkfile(&msg);
				break;
				case MSG_FREAD:
					msg_fread(&msg);
				break;
				case MSG_FWRITE:
					msg_fwrite(&msg);
				break;
			}
			if (sendMessage(sk_cl, &msg) == -1) break;
		}
	}
	/* Chiudo il socket di comunicazione tra client e server */
	closeSocket(sk_cl);
	/* Rimuovo il thread dalla lista dei thread */
	pthread_mutex_lock(&lock);
	num_threads--;
	rimuovi(sk_cl, &my_socket);
	pthread_mutex_unlock(&lock);
	pthread_cond_signal(&wait_me);
	
	return 0;
}

static int msg_mkdir(message_t *msg){
	extern FILE * fs;
	extern struct fat_ctrl f_ctrl;
	extern pthread_mutex_t filesystem;
	int ret_val=0;

	pthread_mutex_lock(&filesystem);
	ret_val = fat_mkdir(fs, &f_ctrl, msg->buffer);	
	pthread_mutex_unlock(&filesystem);
	
	if (msg->buffer) free(msg->buffer);
	msg->length = 0;
	msg->buffer = NULL;
	if (ret_val){
		sfaterror(ret_val);
		msg->type = MSG_ERROR;
	}else msg->type = MSG_OK;

	return ret_val;
}

static int msg_ls(message_t *msg){
	extern FILE * fs;
	extern struct fat_ctrl f_ctrl;
	extern pthread_mutex_t filesystem;
	char *list=NULL;
	int ret_val=0;

	pthread_mutex_lock(&filesystem);
	ret_val = fat_ls(fs, &f_ctrl, msg->buffer, &list);
	pthread_mutex_unlock(&filesystem);
	
	if (msg->buffer) free(msg->buffer);
	if (ret_val){
		if (list) free(list);
		sfaterror(ret_val);
		msg->type = MSG_ERROR;
		msg->length = 0;
		msg->buffer = NULL;
	}else{
		msg->type = MSG_OK;
		msg->length = strlen(list);
		msg->buffer = list;
	}
	return ret_val;
}

static int msg_mkfile(message_t *msg){
	extern FILE * fs;
	extern struct fat_ctrl f_ctrl;	
	extern pthread_mutex_t filesystem;
	int ret_val=0;
	
	pthread_mutex_lock(&filesystem);
	ret_val = fat_open(fs, &f_ctrl, msg->buffer);
	pthread_mutex_unlock(&filesystem);
	
	if (msg->buffer) free(msg->buffer);
	msg->length = 0;
	msg->buffer = NULL;
	if (ret_val){
		sfaterror(ret_val);
		msg->type = MSG_ERROR;
	}else msg->type = MSG_OK;			
	
	return ret_val;
}

static int msg_fread(message_t *msg){
	extern FILE * fs;
	extern struct fat_ctrl f_ctrl;	
	extern pthread_mutex_t filesystem;
	int start=0, data_len=0, ret_val=0;
	char *data=NULL, *path=NULL, *temp=NULL;
	
	/* Memorizzo l'indirizzo di partenza */
	memcpy(&start, msg->buffer, sizeof(int));
	/* Memorizzo la path del file da leggere */
	path = msg->buffer + sizeof(int) + 1;
	/* Memorizzo la dimensione dei dati da leggere */	
	temp = path + strlen(path) + 1;
	memcpy(&data_len, temp, sizeof(int));
	/* Alloco la memoria per i dati da leggere,
	 * "+ 1" perche' termino la stringa */
	data = calloc(data_len+1, sizeof(char));

	pthread_mutex_lock(&filesystem);
	ret_val = fat_read(fs, &f_ctrl, path, start, data_len, data);	
	pthread_mutex_unlock(&filesystem);
	
	if (msg->buffer) free(msg->buffer);
	if (ret_val<0){
		if (data) free(data);
		sfaterror(ret_val);
		msg->type = MSG_ERROR;
		msg->length = 0;
		msg->buffer = NULL;
	}else{
		msg->type = MSG_OK;
		msg->length = strlen(data);
		msg->buffer = data;
	}
	return ret_val;
}

static int msg_fwrite(message_t *msg){
	extern FILE * fs;
	extern struct fat_ctrl f_ctrl;	
	extern pthread_mutex_t filesystem;
	int ret_val=0, data_len=0;
	char *path=NULL, *data=NULL;
	
	path = msg->buffer;
	data = strchr(msg->buffer, '\0') + 1;
	data_len = strlen(data);
	
	pthread_mutex_lock(&filesystem);
	ret_val = fat_write(fs, &f_ctrl, path, data, data_len);
	pthread_mutex_unlock(&filesystem);
	
	if (msg->buffer) free(msg->buffer);
	msg->length = 0;
	msg->buffer = NULL;
	if (ret_val){
		sfaterror(ret_val);
		msg->type = MSG_ERROR;
	}else msg->type = MSG_OK;
	
	return ret_val;
}

