/**
 *   \file		lista.c
 *   \author	Tranchida Giulio, No. Matricola 241732\n
 *				Si dichiara che il contenuto di questo file e',
 * 				in ogni sua parte, opera originale dell'autore.\n\n
 *
 * This program is free software; you can redistribuite it and/or modify
 * it under the terms of the GNU/General Pubblic License as published the
 * Free software Foundation; either version 2 of the License, or
 * (at your opinion) any later version.
 * */
#include "Include/lista.h"

void inizializza(struct lista *stack){
	stack->num_threads=0;	/* azzeramento del contatore */
	stack->top_elem=NULL;	/* azzeramento del puntatore */
}

int ins_new_elem(channel_t sk_cl, struct lista *s){
	struct thread *new = NULL;

	if (!(new = (struct thread *) malloc(sizeof(struct thread)))){
		errore(__FILE__,__LINE__,
		"Errore nel creare un nuovo elemento (STACK)",errno);
		return errno;
	}
	new->sk_cl = sk_cl;
	new->next = s->top_elem;/* fa puntare il nuovo elemento al primo della lista */
	s->top_elem = new;		/* inserisce il nuovo elemento in testa alla lista */
	s->num_threads++;		/* incrementa il valore del contatore */
	return 0;
}

channel_t estrazione(struct lista *s){
	int sk_cl = -1;
	/* Puntatore temporaneo che puntera'
	 * all'area di memoria da eliminare */
	struct thread *tmp;
	
	/* Se la lista non e' vuota... */
	if (s->top_elem){
		sk_cl = s->top_elem->sk_cl;
		/* Ora tmp punta all'area di memoria dell'elemento in cima alla lista */	
		tmp = s->top_elem;
		/* Il secondo elemento della lista diventa il primo,
		 * vengono scambiati cioe' gli indirizzi */
		s->top_elem = s->top_elem->next;
		s->num_threads--;
		free(tmp);
	}
	return sk_cl;
}

/**
 * <h3>delete_elem</h3>
 * 
 * Funzione ricorsiva a solo uso interno, quindi non presente nell'header
 * lista.h, richiamata esclusivamente dalla funzione rimuovi.
 * Rimuove un elemento dall'albero, confrontandolo con il sk_cl passato.
 *
 * \param pun puntatore all'indirizzo del puntatore dell'elemento da cancellare
 * \param sk_cl identificatore del socket, non modificabile
 * 
 * \retval  0 se tutto e' andato bene
 * \retval -1 se l'elemento non e' presente
 * */
int delete_elem(const channel_t sk_cl, struct thread **pun){
	struct thread* nodo = *pun;
	int ret_val = 0;
	
	if (!nodo) return -1;
	if (nodo->sk_cl == sk_cl){
		*pun = nodo->next;
		free(nodo);
		return 0;
	}
	else
		ret_val = delete_elem(sk_cl, &(nodo->next));

	return ret_val;
}

int rimuovi(channel_t sk_cl, struct lista *s){
	int ret_val=0;
	
	if (s->top_elem){
		/* Cerchiamo e rimuoviamo... */
		ret_val = delete_elem(sk_cl, &(s->top_elem));
		if (!ret_val) s->num_threads--;
		return ret_val;
	}
	/* Altrimenti errore lista vuota... */
	return -1;
}

boolean full(const struct lista *s){
	return ((boolean) (s->num_threads == FULL));
}

channel_t leggi_sk_cl(struct lista *s){
	return s->top_elem->sk_cl;
}
