/**
 *	\file		sfat_cp.c
 *	\author		Tranchida Giulio, No Matricola 241732\n
 *				Si dichiara che il contenuto di questo file e',
 * 				in ogni sua parte, opera originale dell'autore.\n\n
 *
 * This program is free software; you can redistribuite it and/or modify
 * it under the terms of the GNU/General Pubblic License as published the
 * Free software Foundation; either version 2 of the License, or
 * (at your opinion) any later version.
 * */

#include "Include/common.h"

/** exit function */
static void byebye();

message_t	msg;				/**< struttura rappresentante un messaggio tra client e server */
channel_t	sk_client;			/**< tipo descrittore del canale di comunicazione (server e client) */
char 		*path;				/**< directory e nome del socket AF_UNIX */
char		*dati_letti=NULL;	/**< buffer temporaneo dei dati letti dal file di origine */

#define LEN	128	/**<< La dimensione che leggero' per ogni ciclo dal file */

/**
 * <h3>Sfat cp</h3>
 * Comando: sfat_cp file1 file2\n \n
 * Crea il file file2 e vi ricopia il contenuto de file file2.\n
 * All'invocazione il client controlla i parametri e contatta il server inviando
 * una richiesta di connessione sul socket.\n Se la connessione ha successo il
 * client invia le opportune richieste di creazione, lettura e scrittura per
 * portare a termine la copia.\n Tutti gli errori sono riportati sullo standar
 * error.  
 * */
int main(int argc, char **argv){
	extern message_t msg;
	extern channel_t sk_client;
	extern char *path;
	extern char *dati_letti;
	int dest_len=0, source_len=0, offset=0, len=LEN, letti=0;
	char *buffer=NULL;
	
	sk_client = 0;
	path = NULL;
	msg.buffer = NULL;
	
	if ((atexit(byebye))!=0){
    	errore(__FILE__,__LINE__,"sfat_cp: cannot set exit function",errno);
    	exit(EXIT_FAILURE);
	}
	
	/* Argomenti da riga di comando */
	if (argc<3){
		fprintf(stderr,"sfat_cp: errore argomenti insufficenti\n");
		fprintf(stderr,"./sfat_cp file_src file_dest\n");
		exit(EXIT_FAILURE);
	}	
	
	if (!(path = calloc((strlen(TMP) + strlen(SKTNAME) + 1), sizeof(char)))){
		errore(__FILE__,__LINE__,
		"sfat_cp: error on allocate memory for 'path'",errno);
		exit(EXIT_FAILURE);
	}
	strncpy(path, TMP, strlen(TMP) + 1);
	strncat(path, SKTNAME, strlen(SKTNAME));

	/* Apro la socket di comunicazione col server */
	if ((sk_client = openConnection(path)) == -1) {
		fprintf(stderr,"sfat_cp: Errore nella apertura del socket di comunicazione\n");
		exit(EXIT_FAILURE);
	}
	else if (sk_client == SFATENAMETOOLONG){
		fprintf(stderr,"sfat_cp: Error Path Too Long (exceeding UNIX_PATH_MAX)\n");    	
		exit(EXIT_FAILURE);
	}

	dest_len = strlen(argv[2]) + 1;
	source_len = strlen(argv[1]) + 1;

	/* Creo il file */
	msg.type = MSG_MKFILE;
	msg.length = dest_len;
	if (!(msg.buffer = strdup(argv[2]))){
		errore(__FILE__,__LINE__,
		"sfat_cp: error on allocate memory for 'buffer'",errno);
		exit(EXIT_FAILURE);
	}
	if (sendMessage(sk_client, &msg) == -1) exit(EXIT_FAILURE);
	if (receiveMessage(sk_client, &msg) == -1) exit(EXIT_FAILURE);
	if (msg.type==MSG_ERROR) exit(EXIT_FAILURE);
	
	while(1){
		/* Leggo il file da copiare */
		msg.type = MSG_FREAD;	
		msg.length = 2*sizeof(int) + source_len*sizeof(char) + 2;
		if (!(msg.buffer = malloc(msg.length))){
			errore(__FILE__,__LINE__,
			"sfat_cp: error on allocate memory for 'buffer'",errno);
			exit(EXIT_FAILURE);
		}
		memset(msg.buffer, 0x0, msg.length);
		/* Struttura del messaggio: Offset\0Path\0Len\0 */
		memcpy(msg.buffer, &offset, sizeof(int));
		buffer = msg.buffer + sizeof(int) + 1;
		strncpy(buffer, argv[1], source_len);
		buffer += source_len;
		memcpy(buffer, &len, sizeof(int));
		
		/* Mando la richiesta di lettura al server */
		if (sendMessage(sk_client, &msg) == -1) exit(EXIT_FAILURE);
		if (receiveMessage(sk_client, &msg) == -1) exit(EXIT_FAILURE);
		if (msg.type==MSG_ERROR) exit(EXIT_FAILURE);
		
		/* Mi faccio una copia dei dati letti */
		if (!(dati_letti = strdup(msg.buffer))){
			errore(__FILE__,__LINE__,
			"sfat_cp: error on allocate memory for 'buffer'",errno);
			exit(EXIT_FAILURE);		
		}
		
		/* Preparo il messaggio da inviare al server */
		msg.type = MSG_FWRITE;
		msg.length = msg.length + strlen(argv[2]) + 2;
		if (msg.buffer) free(msg.buffer);
		if (!(msg.buffer = calloc(msg.length, sizeof(char)))){
			errore(__FILE__,__LINE__,
			"sfat_cp: error on allocate memory for 'buffer'",errno);
			exit(EXIT_FAILURE);
		}
		/* Struttura del messaggio: Path\0Buf\0 */
		strncpy(msg.buffer, argv[2], strlen(argv[2]));
		buffer = msg.buffer + dest_len;
		strncpy(buffer, dati_letti, strlen(dati_letti));
		letti = strlen(dati_letti);
		if (dati_letti){
			free(dati_letti);
			dati_letti = NULL;
		}
		/* Mando la richiesta di scrittura al server */
		if (sendMessage(sk_client, &msg) == -1) exit(EXIT_FAILURE);
		if (receiveMessage(sk_client, &msg) == -1) exit(EXIT_FAILURE);
		if (msg.type==MSG_ERROR) exit(EXIT_FAILURE);
		
		if (letti<len) break;
		else offset += len;		
	}
	if (msg.type==MSG_OK) printf("sfat_cp: %s written \n", argv[2]);
	else exit(EXIT_FAILURE);
	
	exit(EXIT_SUCCESS);
	return 0;     
}

void byebye(){
	extern message_t msg;
	extern char * path;
	extern char *dati_letti;
	extern channel_t sk_client;
	
	/* Chiudo la socket di comunicazione del client */
	if (sk_client != -1) closeConnection(sk_client);
			
	if (msg.buffer!=NULL) free(msg.buffer);
	if (dati_letti) free(dati_letti);
	if (path!=NULL) free(path);
}
