/**
 * 	\file		sfat_create.c
 *	\author		Tranchida Giulio, No Matricola 241732\n
 *				Si dichiara che il contenuto di questo file e',
 * 				in ogni sua parte, opera originale dell'autore.\n\n
 *
 * This program is free software; you can redistribuite it and/or modify
 * it under the terms of the GNU/General Pubblic License as published the
 * Free software Foundation; either version 2 of the License, or
 * (at your opinion) any later version.
 * */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/stat.h>
#include <limits.h>

#include "../../Fat/Src/Include/format.h"
#include "../../Fat/Src/Include/sfaterror.h"
#include "../../Fat/Src/Include/fat_defs.h"

/**
 * <h3>Sfat create</h3>
 * Comando: sfat_create device_name nblock size\n \n
 * Crea un file di nome device_name da usare come device per un filesystem
 * sfat e lo formatta in modo che la data region contenga nblocks blocchi
 * di ampiezza size. Ricordando che size puo' assumere uno dei 4 valori: 128,
 * 1K, 2K, 4K.\n
 * Se il file esiste la funzione termina con errore. 
 * */
int main(int argc, char **argv){
	FILE *fs = NULL;
	char *endptr;
	int block_size=0;			/**< Block Size (bytes) */
	unsigned int num_block=0;	/**< Numero di blocchi data region */
	struct stat buf;
	
	/* Argomenti da riga di comando */
	if (argc<2){
		fprintf(stderr,"sfat_create: Errore nome filesystem mancante\n");
		exit(EXIT_FAILURE);
	}
	else if (argc<3){
		fprintf(stderr,"sfat_create: Errore numero di blocchi mancante\n");		
		exit(EXIT_FAILURE);
	}
	else if (argc<4){
		fprintf(stderr,"sfat_create: Errore dimensione dei blocchi mancante\n");		
		exit(EXIT_FAILURE);
	}
	else if (lstat(argv[1], &buf) == 0){
		fprintf(stderr,"sfat_create: Errore file gia' esistente\n");		
		exit(EXIT_FAILURE);
	}

	num_block = strtoul(argv[2], &endptr, 10);	/* unsigned */
	if ((errno == ERANGE) || (errno != 0 && num_block == 0)) {
		perror("Strtoul");
		exit(EXIT_FAILURE);
	}
	if (endptr == argv[2]) {
		fprintf(stderr, "sfat_create: No digits were found\n");
		exit(EXIT_FAILURE);
	}
	if (num_block <= 0){
		fprintf(stderr, "sfat_create: error number blocks\n");
		exit(EXIT_FAILURE);    
	}
    
	block_size = strtol(argv[3], &endptr, 10);
	if (!((block_size==128) || (block_size==1024) || (block_size==2048) || (block_size==4096))){
		fprintf(stderr, "sfat_create: bad size block\n");
		return EWFCD;						/* bad size block */
	}
	
	if (!(fs = fopen(argv[1],"w"))){
		perror(NULL);
		exit(EXIT_FAILURE);
	}
	
	if (EWFCD == fat_format(fs, block_size, num_block)){
		sfaterror (EWFCD);
		if (fclose(fs)) perror(NULL); 
		exit(EXIT_FAILURE);
	}
	printf("sfat_create: Sto creando %s di %d blocchi size %d\n", argv[1], num_block, block_size);

	if (fclose(fs)){
		perror(NULL);
		exit(EXIT_FAILURE);
	}
	exit(EXIT_SUCCESS);
 	return 0;   
}
