/**
 * 	\file		sfat_mkdir.c
 *	\author		Tranchida Giulio, No Matricola 241732\n
 *				Si dichiara che il contenuto di questo file e',
 * 				in ogni sua parte, opera originale dell'autore.\n\n
 *
 * This program is free software; you can redistribuite it and/or modify
 * it under the terms of the GNU/General Pubblic License as published the
 * Free software Foundation; either version 2 of the License, or
 * (at your opinion) any later version.
 * */
#include "Include/common.h"

/** exit function */
static void byebye();

message_t	msg;		/**< struttura rappresentante un messaggio tra client e server */
channel_t	sk_client;	/**< tipo descrittore del canale di comunicazione (server e client) */
char 		*path;		/**< directory e nome del socket AF_UNIX */

/**
 * <h3>Sfat mkdir</h3>
 * Comando: sfat_mkdir dir1/.../dirk/dir \n \n
 * crea una directory "dir" nella directory "dir1/.../dirk/", se non esiste alcun
 * file o directory con lo stesso nome. Tutte le directory "dir1/.../dirk/"
 *  devono esistere.\n  
 * All'invocazione il client controlla i parametri e contatta il server inviando
 * una richiesta di connessione sul socket.\n Se la connessione ha successo il
 * client invia una richiesta di MSG_MKDIR.\n Tutti gli errori sono riportati
 * sullo standar error.  
 * */
int main(int argc, char **argv){
	extern message_t msg;
	extern channel_t sk_client;
	extern char * path;
	
	sk_client = 0;
	path = NULL;
	msg.buffer = NULL;
	
	if ((atexit(byebye))!=0){
		errore(__FILE__,__LINE__,"sfat_mkdir: cannot set exit function",errno);
		exit(EXIT_FAILURE);
	}
	
	/* Argomenti da riga di comando */
	if (argc<2){
		fprintf(stderr,
		"sfat_mkdir: errore non e' stato passato alcun nome per la cartella da creare\n");
		exit(EXIT_FAILURE);
	}
	
	if (!(path = calloc((strlen(TMP) + strlen(SKTNAME) + 1), sizeof(char)))){
		errore(__FILE__,__LINE__,
		"sfat_mkdir: error on allocate memory for 'path'",errno);
		exit(EXIT_FAILURE);
	}
	strncpy(path,TMP,strlen(TMP));
	strncat(path,SKTNAME,strlen(SKTNAME));
    	
	msg.type = MSG_MKDIR;
	msg.length = strlen(argv[1])+1;
	if (!(msg.buffer = malloc(msg.length))){
		errore(__FILE__,__LINE__,
		"sfat_mkdir: error on allocate memory for 'buffer'",errno);
		exit(EXIT_FAILURE);
	}
	memset(msg.buffer, 0x0, msg.length);
	/* uso strncat perche' termina la stringa */
	strncat(msg.buffer,argv[1],msg.length);
	
	/* Apro la socket di comunicazione col server */
	if ((sk_client = openConnection(path)) == -1) {
		fprintf(stderr,"sfat_read: Errore nella apertura del socket di comunicazione\n");
		exit(EXIT_FAILURE);
	}
	else if (sk_client == SFATENAMETOOLONG){
		fprintf(stderr,"sfat_read: Error Path Too Long (exceeding UNIX_PATH_MAX)\n");    	
		exit(EXIT_FAILURE);
	}

	if (sendMessage(sk_client, &msg) == -1) exit(EXIT_FAILURE);
	if (receiveMessage(sk_client, &msg) == -1) exit(EXIT_FAILURE);
	if (msg.type==MSG_OK)
		printf("sfat_mkdir: %s created!\n", argv[1]);
	else exit(EXIT_FAILURE);

	exit(EXIT_SUCCESS);
	return 0;     
}

void byebye(){
	extern message_t msg;
	extern char * path;
	extern channel_t sk_client;
	
	/* Chiudo la socket di comunicazione del client */
	if (sk_client != -1) closeConnection(sk_client);
	
	if (msg.buffer!=NULL) free(msg.buffer);
	if (path!=NULL) free(path);
}
