/**
 *	\file		sfat_read.c
 *	\author		Tranchida Giulio, No Matricola 241732\n
 *				Si dichiara che il contenuto di questo file e',
 * 				in ogni sua parte, opera originale dell'autore.\n\n
 *
 * This program is free software; you can redistribuite it and/or modify
 * it under the terms of the GNU/General Pubblic License as published the
 * Free software Foundation; either version 2 of the License, or
 * (at your opinion) any later version.
 * */

#include <sys/stat.h>
#include "Include/common.h"

/** exit function */
static void byebye();

message_t	msg;		/**< struttura rappresentante un messaggio tra client e server */
channel_t	sk_client;	/**< tipo descrittore del canale di comunicazione (server e client) */
char 		*path;		/**< directory e nome del socket AF_UNIX */

/**
 * <h3>Sfat read</h3>
 * Comando: sfat_read path offset len legge al piu' len byte dal file
 * a partire dall’offset.\n
 * All’invocazione il client controlla i parametri contatta il server
 * inviando una richesta di connessione sul socket.\n Se la connessione
 * ha successo il client invia una richiesta di MSG_FREAD.\n  Tutti gli
 * errori sono riportati sullo standar error.  
 * 
 * */
int main(int argc, char **argv){
	extern message_t msg;
	extern channel_t sk_client;
	extern char * path;
	char *endptr, *buffer;
	int offset=0, len=0;
	
	sk_client = 0;
	path = NULL;
	msg.buffer = NULL;
	
	if ((atexit(byebye))!=0){
		errore(__FILE__,__LINE__,"sfat_read: cannot set exit function",errno);
		exit(EXIT_FAILURE);
	}
	
	/* Argomenti da riga di comando */
	if (argc<4){
		fprintf(stderr,"sfat_read: error too few arguments\n");
		exit(EXIT_FAILURE);
	}
	/* OFFSET */
	offset = strtol(argv[2], &endptr, 10);
	if (errno!=0){
		errore(__FILE__,__LINE__,"sfat_read: No digits were found (offset)",errno);
		exit(EXIT_FAILURE);
	}
	if (endptr == argv[2]){
		fprintf(stderr, "sfat_read: No digits were found (offset)\n");
		exit(EXIT_FAILURE);
	}
	if (offset < 0){
		fprintf(stderr, "sfat_read: error negative offset number\n");
		exit(EXIT_FAILURE);    
	}
	/* LEN */
	len = strtol(argv[3], &endptr, 10);
	if (errno!=0){
		errore(__FILE__,__LINE__,"sfat_read: No digits were found (len)",errno);
		exit(EXIT_FAILURE);
	}
	if (endptr == argv[3]){
		fprintf(stderr, "sfat_read: No digits were found (len)\n");
		exit(EXIT_FAILURE);
	}
	if (len <= 0){
		fprintf(stderr, "sfat_read: error in len number\n");
		exit(EXIT_FAILURE);    
	}
	
	/* Path del socket */
	if (!(path = calloc((strlen(TMP) + strlen(SKTNAME) + 1), sizeof(char)))){
		errore(__FILE__,__LINE__,
		"sfat_read: error on allocate memory for 'path'",errno);
		exit(EXIT_FAILURE);
	}
	strncpy(path, TMP, strlen(TMP)+1);
	strncat(path, SKTNAME, strlen(SKTNAME));
	
	/* Preparo il messaggio da inviare al server */
	msg.type = MSG_FREAD;
	msg.length = sizeof(int)*2  + strlen(argv[1]) + 3;
	if (!(msg.buffer = malloc(msg.length))){
		errore(__FILE__,__LINE__,
		"sfat_read: error on allocate memory for 'buffer'",errno);
		exit(EXIT_FAILURE);
	}
	memset(msg.buffer, 0x0, msg.length);
	/* Struttura del messaggio: Offset\0Path\0Len\0 */
	memcpy(msg.buffer, &offset, sizeof(int));
	buffer = msg.buffer + sizeof(int) + 1;
	strncpy(buffer,argv[1],strlen(argv[1]) + 1);
	buffer += strlen(argv[1]) + 1;
	memcpy(buffer, &len, sizeof(int));

	/* Apro la socket di comunicazione col server */
	if ((sk_client = openConnection(path)) == -1) {
		fprintf(stderr,"sfat_read: Errore nella apertura del socket di comunicazione\n");
		exit(EXIT_FAILURE);
	}
	else if (sk_client == SFATENAMETOOLONG){
		fprintf(stderr,"sfat_read: Error Path Too Long (exceeding UNIX_PATH_MAX)\n");    	
		exit(EXIT_FAILURE);
	}  
	if (sendMessage(sk_client, &msg) == -1) exit(EXIT_FAILURE);
	if (receiveMessage(sk_client, &msg) == -1) exit(EXIT_FAILURE);
	if (msg.type==MSG_OK && msg.length!=0)
		printf("sfat_read: read: %s \"%s\"\n", argv[1], msg.buffer);	
	else exit(EXIT_FAILURE);
	
	exit(EXIT_SUCCESS);
	return 0;     
}

void byebye(){
	extern message_t msg;
	extern char * path;
	extern channel_t sk_client;
	
	/* Chiudo la socket di comunicazione del client */
	if (sk_client != -1) closeConnection(sk_client);
			
	if (msg.buffer!=NULL) free(msg.buffer);
	if (path!=NULL) free(path);
}
