/**
 * 	\file		format.c
 *	\author		Tranchida Giulio, No Matricola 241732\n
 *				Si dichiara che il contenuto di questo file e',
 * 				in ogni sua parte, opera originale dell'autore.\n\n
 *
 * This program is free software; you can redistribuite it and/or modify
 * it under the terms of the GNU/General Pubblic License as published the
 * Free software Foundation; either version 2 of the License, or
 * (at your opinion) any later version.
 * */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "Include/fat.h"
#include "Include/format.h"

int fat_format(FILE *fs, int block_size, unsigned int num_block){
	struct fat_ctrl myfat;
	void * data_r_block;
	struct dir_entry cart_p;
	int i, conta=0;

	if ( !(fs) || !(block_size) || !(num_block)) return EBDP;
    if (!((block_size==128) || (block_size==1024) || (block_size==2048)
    	|| (block_size==4096)))
    	return EWFCD;

	/* - Inizializzazione del boot sector. */
	myfat.b_sector.block_size = block_size;
	myfat.b_sector.fs_type = FAT_TYPE;
	myfat.b_sector.num_block = num_block;
	myfat.blk_base = sizeof(struct boot_sector) + sizeof(unsigned int) * num_block;
	
	/* - Scrittura dei dati nel boot sector. */
	if (!(fwrite(&myfat.b_sector, sizeof(struct boot_sector), 1, fs)))
		return EWFCD;
	
	/* - Inizializzazione della File Allocation Table.
	 * (sarebbe bastata un calloc, ma se a BLOCK_FREE si cambia valore,
	 * l'errore sarebbe piuttosto evidente)
	 * */
	if (!(myfat.fat_ptr = malloc(num_block * sizeof(unsigned int))))
		return EWFCD;

	for (i=1; i<myfat.b_sector.num_block; i++)
		myfat.fat_ptr[i] = BLOCK_FREE;

	myfat.fat_ptr[ROOT_IDX] = LAST_BLOCK;
	
	/* Scrittura dei dati della FAT nel file */
	if (!(fwrite(myfat.fat_ptr, (num_block * sizeof(unsigned int)), 1, fs)))
		return EWFCD;

	free(myfat.fat_ptr);
		
	/* Creazione del primo blocco della DATA REGION */
	if (!(data_r_block = malloc(block_size))) return EWFCD;

	/* Inizializzazione del blocco della Data Region */
	memset(data_r_block, INIT_BLOCK, block_size);
	
	/* Scrittura dei dati della DATA REGION nel file */
	for (i=0; i<num_block; i++)
		conta += fwrite(data_r_block, block_size, 1, fs);
	if (conta != num_block) return EWFCD;
	free(data_r_block);
	
	/* Torno alla root (inizio data region)*/
	fseek(fs, myfat.blk_base, SEEK_SET);
	
	memset(cart_p.name, 0x0, MAX_LEN_NAME+1);
	/* Il primo blocco della data region conterra' la Directory Table della radice */
	/* fill info about dir entry '.' and '..' */
	cart_p.attr = SUB_ENTRY;
	cart_p.index = ROOT_IDX;
	cart_p.len = 2 * sizeof(struct dir_entry);
	strncpy(cart_p.name,".",MAX_LEN_NAME);	
	cart_p.used = DIR_ENTRY_BUSY;

	if (!(fwrite(&cart_p, sizeof(struct dir_entry), 1, fs))) return EWFCD;

	strncpy(cart_p.name,"..",MAX_LEN_NAME+1);
	if (!(fwrite(&cart_p, sizeof(struct dir_entry), 1, fs))) return EWFCD;

	return 0;	
}
