/**
 *	\file		list.c
 *	\author		Tranchida Giulio, No Matricola 241732\n
 *				Si dichiara che il contenuto di questo file e',
 * 				in ogni sua parte, opera originale dell'autore.\n\n
 *
 * This program is free software; you can redistribuite it and/or modify
 * it under the terms of the GNU/General Pubblic License as published the
 * Free software Foundation; either version 2 of the License, or
 * (at your opinion) any later version.
 * */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "Include/fat.h"
#include "Include/list.h"
#include "Include/util.h"

int fat_ls(FILE *fs, struct fat_ctrl *f_ctrl, char *path, char** list){
	struct dir_entry cart_p;
	int i, j, ret_val, numero_entry, MAX_ENTRY;
	unsigned int size, indice_fat;
	char ** lista;
	long fs_pos;
	char * list_tmp;
		
	if ( !(fs) || !(f_ctrl) || !(path) ) return EBDP;
	/* La funzione parsing mi lascia il file pointer fs all'inzio del
	 * primo blocco della DATA REGION che contiene la prima porzione 
	 * della Directory Table relativa alla cartella puntata dalla path,
	 * se esiste, altrimenti mi ritorna un codice di errore. */
	if ((ret_val = parsing(fs, f_ctrl, path, LS))!=0) return ret_val;
	MAX_ENTRY = f_ctrl->b_sector.block_size/sizeof(struct dir_entry);
	
	/* Leggo la dir_entry "." */
	if (!(fread(&cart_p, sizeof(struct dir_entry), 1, fs)))	return ERBD;

	numero_entry = cart_p.len/sizeof(struct dir_entry);
	indice_fat = cart_p.index;
	if (indice_fat > f_ctrl->b_sector.num_block) return ERFCD;
	size = 2;

	if (!(lista = calloc(numero_entry, sizeof(char *)))) return STDLIBERR;
	if (!(lista[0] = strdup(cart_p.name))) return STDLIBERR;
	
	for (i=1, j=1; i<numero_entry; i++, j++){
		if (j==MAX_ENTRY){
			if (f_ctrl->fat_ptr[indice_fat]==BLOCK_FREE) return ERFCD;
			fs_pos = f_ctrl->blk_base + (f_ctrl->fat_ptr[indice_fat] * f_ctrl->b_sector.block_size);
			fseek(fs, fs_pos, SEEK_SET);
			indice_fat = f_ctrl->fat_ptr[indice_fat];
			j=1;
		}
		if (!(fread(&cart_p, sizeof(struct dir_entry), 1, fs))) return ERBD;
		if (!strlen(cart_p.name)) return ERBD;
		size += strlen(cart_p.name)+1;
		if (!(lista[i] = strdup(cart_p.name))) return STDLIBERR;
	}

	if (!(list_tmp = calloc(size, sizeof(char *)))) return STDLIBERR;
	for (i=0; i<numero_entry; i++){
	    if (i<numero_entry-1){
			strcat(list_tmp, lista[i]);
			strcat(list_tmp, "\t");
			free(lista[i]);
	    }else{
			strcat(list_tmp, lista[i]);
			strcat(list_tmp, "\0");
			free(lista[i]);
	    }
	}
	free(lista);
	
	/* SP ritorna il puntatore alla lista dei file */
	*list=list_tmp;	

	return ret_val;
}
