/**
 *	\file		make_dir.c
 *	\author		Tranchida Giulio, No Matricola 241732\n
 *				Si dichiara che il contenuto di questo file e',
 * 				in ogni sua parte, opera originale dell'autore.\n\n
 *
 * This program is free software; you can redistribuite it and/or modify
 * it under the terms of the GNU/General Pubblic License as published the
 * Free software Foundation; either version 2 of the License, or
 * (at your opinion) any later version.
 * */
#include <stdio.h>
#include <libgen.h>
#include <stdlib.h>
#include <string.h>

#include "Include/fat.h" 
#include "Include/util.h"

int fat_mkdir(FILE *fs, struct fat_ctrl *f_ctrl, char *path){
	struct dir_entry cart_p;
	struct dir_entry cart_new;
	char *basec, *bname;
	int ret_val=0;
	unsigned int new_index=0;
	long fs_radice;
	
	if ( !(fs) || !(f_ctrl) || !(path) ) return EBDP;
	if (!strcmp("/", path)) return EDAEX;

	/* a) Posizionamento nel filesystem, controllo del percoso ecc. */
	if ((ret_val = parsing(fs, f_ctrl, path, MKDIR))!=0) return ret_val;

	/* Reperisco il nome della cartella da creare */
	if (!(basec = strdup(path))) return STDLIBERR;
	bname = basename(basec);
	if (!strcmp(".", bname) || !strcmp("..", bname)){free(basec); return EDAEX;}
		
	fs_radice = ftell(fs);
	/* Leggo l'entry iniziale '.' della cartella corrente */
	if (!(fread(&cart_p, sizeof(struct dir_entry), 1, fs))){free(basec); return ERBD;}

	/* Queste due righe sottostanti vanno sempre insieme!! */
	fseek(fs, fs_radice, SEEK_SET);
	/*	b) controllo che la cartella non esista gia' */
	ret_val = find(fs, f_ctrl, bname, MKDIR);
	
	/* Cerco un nuovo blocco libero della fat (BLOCK_FREE) che verra' dedicato
	 alle entry della nuova cartella. se non c'e' ne sono esco con ENMSD */
	if (ret_val==0){ 
		for (new_index=0; new_index<f_ctrl->b_sector.num_block; new_index++)
			if (f_ctrl->fat_ptr[new_index] == BLOCK_FREE) break;
	}
	if (new_index>=f_ctrl->b_sector.num_block || ret_val!=0){
		/* Al fine di annullare i possibili cambiamenti portati dalla funzione
		 * 'find' rileggo le informazioni della fat */
		fseek(fs, sizeof(struct boot_sector), SEEK_SET);
		if (!(fread(f_ctrl->fat_ptr, (f_ctrl->b_sector.num_block * sizeof(unsigned int)), 1, fs)))
			ret_val = EWFCD;
		else if (ret_val==0) ret_val = ENMSD; 
		free(basec);
		return ret_val;
	}

	/* Il blocco libero trovato della FAT deve essere impostato ad LAST_BLOCK. */
	f_ctrl->fat_ptr[new_index] = LAST_BLOCK;

	cart_new.attr = SUB_ENTRY;
	cart_new.index = new_index;		/* Indice del blocco delle entry della nuova cartella */
	cart_new.len = 2 * sizeof(struct dir_entry);
	memset(cart_new.name, 0x0, MAX_LEN_NAME+1);
	strncpy(cart_new.name,bname,MAX_LEN_NAME);
	cart_new.used = DIR_ENTRY_BUSY;

	/* Scrivo l'entry della cartella che sto creando, nella sua cartella radice */
	if (!(fwrite(&cart_new, sizeof(struct dir_entry), 1, fs))){free(basec); return EWFCD;}

	strncpy(cart_new.name,".",MAX_LEN_NAME);
	/* Posiziono il file pointer fs all'inizio del blocco che conterra' tutte le entry
	 * relative alla cartella che sto creando */
	fseek(fs, (f_ctrl->blk_base + (cart_new.index * f_ctrl->b_sector.block_size)), SEEK_SET);	
	if (!(fwrite(&cart_new, sizeof(struct dir_entry), 1, fs))){free(basec); return EWFCD;}

	cart_new.index = cart_p.index;
	strncpy(cart_new.name,"..",MAX_LEN_NAME);
	if (!(fwrite(&cart_new, sizeof(struct dir_entry), 1, fs))){free(basec); return EWFCD;}
	
	/* Devo incrementare il conteggio delle entry nella cartella che contiene
	la nuova cartella che sto creando e scrivero' questo nuovo dato nel filesystem */
	fseek(fs, fs_radice, SEEK_SET);
	cart_p.len += sizeof(struct dir_entry);
	if (!(fwrite(&cart_p, sizeof(struct dir_entry), 1, fs))){free(basec); return ERBD;}

	/* Metto il puntatore fs all'inizio della FAT */
	fseek(fs, sizeof(struct boot_sector), SEEK_SET);	
	/* Scrittura dei nuovi dati della FAT nel file */
	if (!(fwrite(f_ctrl->fat_ptr, (f_ctrl->b_sector.num_block * sizeof(unsigned int)), 1, fs)))
		ret_val = EWFCD;
	
	free(basec);
	return ret_val;
}
