/**
 *	\file		open_fat.c
 *	\author		Tranchida Giulio, No Matricola 241732\n
 *				Si dichiara che il contenuto di questo file e',
 * 				in ogni sua parte, opera originale dell'autore.\n\n
 *
 * This program is free software; you can redistribuite it and/or modify
 * it under the terms of the GNU/General Pubblic License as published the
 * Free software Foundation; either version 2 of the License, or
 * (at your opinion) any later version.
 * */

#include <stdio.h> 
#include <stdlib.h>
#include <libgen.h>
#include <string.h>

#include "Include/fat.h"
#include "Include/util.h"

int fat_open(FILE *fs, struct fat_ctrl *f_ctrl, char *path){
	struct dir_entry cart_p, file_new;
	char *basec, *bname;
	int ret_val=0;
	long fs_radice;
		
	if (!(fs) || !(f_ctrl) || !(path)) return EBDP;
	if (!strcmp("/", path)) return EDAEX;
	/* a) Posizionamento nel filesystem, controllo del percoso ecc. */
	if ((ret_val = parsing(fs, f_ctrl, path, MKFILE))!=0) return ret_val;

	/* Reperisco il nome del file da creare */
	if (!(basec = strdup(path))) return STDLIBERR;
	bname = basename(basec);
	if (!strcmp(".", bname) || !strcmp("..", bname)){free(basec); return EDAEX;}
	
	fs_radice = ftell(fs);
	/* Leggo l'entry iniziale '.' della cartella corrente */
	if (!(fread(&cart_p, sizeof(struct dir_entry), 1, fs))){free(basec); return ERBD;}

	/* Queste due righe sottostanti vanno sempre insieme!! */
	fseek(fs, fs_radice, SEEK_SET);
	/* Controllo che il file non esista gia' */
	if ((ret_val = find(fs, f_ctrl, bname, MKFILE))!=0){free(basec); return ret_val;}

	file_new.attr = FILE_ENTRY;
	file_new.index= BLOCK_FREE;	/* Indice del blocco del nuovo file ?? */
	file_new.len = 0;
	memset(file_new.name, 0x0, MAX_LEN_NAME+1);
	strncpy(file_new.name,bname,MAX_LEN_NAME);
	file_new.used = DIR_ENTRY_BUSY;
	
	/* Scrivo l'entry del file che sto creando, nella sua cartella radice */
	if (!(fwrite(&file_new, sizeof(struct dir_entry), 1, fs))){free(basec); return EWFCD;}
	
	/* Devo incrementare il conteggio delle entry nella cartella che contiene
	il nuovo file che sto creando e scrivero' questo nuovo dato nel filesystem */
	fseek(fs, fs_radice, SEEK_SET);
	cart_p.len += sizeof(struct dir_entry);
	if (!(fwrite(&cart_p, sizeof(struct dir_entry), 1, fs))){free(basec); return ERBD;}

	/* Metto il puntatore fs all'inizio della FAT */
	fseek(fs, sizeof(struct boot_sector), SEEK_SET);	
	/* Scrittura dei nuovi dati della FAT nel file */
	if (!(fwrite(f_ctrl->fat_ptr, (f_ctrl->b_sector.num_block * sizeof(unsigned int)), 1, fs)))
		ret_val = EWFCD;
	
	free(basec);		
	return ret_val;
}
