/**
 *	\file		read_fat.c
 *	\author		Tranchida Giulio, No Matricola 241732\n
 *				Si dichiara che il contenuto di questo file e',
 * 				in ogni sua parte, opera originale dell'autore.\n\n
 *
 * This program is free software; you can redistribuite it and/or modify
 * it under the terms of the GNU/General Pubblic License as published the
 * Free software Foundation; either version 2 of the License, or
 * (at your opinion) any later version.
 * */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "Include/fat.h" 
#include "Include/util.h"

int fat_read(FILE *fs, struct fat_ctrl *f_ctrl, char *path, int start, 
	     int data_len, char *data){
	struct dir_entry file;
	int letti=0, to_read=0, my_start=start, len=data_len, byte_da_leggere=0;
	unsigned int index=0;
	char* dati = data;
	long fs_pos=0;
	
	if (!(fs) || !(f_ctrl) || !(path)) return EBDP;
	
	/* a) Posizionamento nel filesystem, controllo del percoso, ecc. */
	if ((letti = parsing(fs, f_ctrl, path, RDFILE))!=0) return letti;
	
	/* Leggo l'entry del file */
	if (!(fread(&file, sizeof(struct dir_entry), 1, fs))) return ERBD;
	if ((file.len==0) || (start>file.len) || (data_len==0)) return 0;
	if (data_len>file.len) len=file.len;
	if ((start + len) > file.len)
		len -= ((start + len) - file.len);

	/* Calcolo la giusta posizione di partenza per 'fs' */
	index = file.index;
	while(my_start>=f_ctrl->b_sector.block_size){
		my_start -= f_ctrl->b_sector.block_size;
		if ((my_start>f_ctrl->b_sector.block_size) && (f_ctrl->fat_ptr[index]==LAST_BLOCK))
			return ERFCD;
		index = f_ctrl->fat_ptr[index]; 
	}
	fs_pos = f_ctrl->blk_base + my_start + (index * f_ctrl->b_sector.block_size);
	fseek(fs, fs_pos, SEEK_SET);
	
	byte_da_leggere = len;
	while(len){
		if ((my_start+len) > f_ctrl->b_sector.block_size){
			to_read = f_ctrl->b_sector.block_size - my_start;
			len -= to_read;
			my_start = 0;
		}
		else if ((my_start+len) <= f_ctrl->b_sector.block_size){
			to_read = len;
		}
		if (!(fread(dati, to_read, 1, fs))) return ERBD;
		dati += to_read; letti += to_read;
		if (letti == byte_da_leggere) break;
		/* Altrimenti spostati nel blocco successivo della FAT */
		if (f_ctrl->fat_ptr[index]==BLOCK_FREE)	return ERFCD;
		fs_pos = f_ctrl->blk_base + (f_ctrl->fat_ptr[index] * f_ctrl->b_sector.block_size);
		fseek(fs, fs_pos, SEEK_SET);
		index = f_ctrl->fat_ptr[index];
	}
	return letti;
}
