/**
 *	\file		write_fat.c
 *	\author		Tranchida Giulio, No Matricola 241732\n
 *				Si dichiara che il contenuto di questo file e',
 * 				in ogni sua parte, opera originale dell'autore.\n\n
 *
 * This program is free software; you can redistribuite it and/or modify
 * it under the terms of the GNU/General Pubblic License as published the
 * Free software Foundation; either version 2 of the License, or
 * (at your opinion) any later version.
 * */
 
#include <stdio.h> 
#include <stdlib.h>
#include <libgen.h>
#include <string.h>

#include "Include/fat.h"
#include "Include/util.h"

int fat_write(FILE *fs, struct fat_ctrl *f_ctrl, char *path, char* data, int data_len){
	struct dir_entry file;
	int ret_val=0, index=0, new_index=0;
	int dati_len=0, start_index=0, da_scrivere=0;
	unsigned int len, my_start;
	long fs_entry, fs_pos=0;
	char * dati;
	
	if (!(fs) || !(f_ctrl) || !(path)) return EBDP;
	if (!strcmp("/", path)) return EDAEX;
	if (data_len==0 || data==NULL) return 0;
	
	/* a) Posizionamento nel filesystem, controllo del percoso ecc. */
	if ((ret_val = parsing(fs, f_ctrl, path, WRFILE))!=0) return ret_val;
	
	fs_pos = fs_entry = ftell(fs);
	/* Leggo l'entry del file */
	if (!(fread(&file, sizeof(struct dir_entry), 1, fs))) return ERBD;
	
	len = file.len; index = file.index;
	/* Caso pessimo, il file contiene gia' informazioni,
	 * scorro la FAT fino ad raggiungere l'ultimo blocco del file */	
	while (len > f_ctrl->b_sector.block_size){
		if (f_ctrl->fat_ptr[index]==BLOCK_FREE || f_ctrl->fat_ptr[index]==LAST_BLOCK)
			return ERFCD;
		index = f_ctrl->fat_ptr[index];			
		len -= f_ctrl->b_sector.block_size;
		fs_pos = f_ctrl->blk_base + (index * f_ctrl->b_sector.block_size);
	}
	if ((len<f_ctrl->b_sector.block_size) && (len!=0)){
		fs_pos = f_ctrl->blk_base + (index * f_ctrl->b_sector.block_size);
		fs_pos += len;
	}
	if (len == f_ctrl->b_sector.block_size) len = 0; 
	if (len == 0){
		/* Qui devo allocare un nuovo blocco e posizionarmi all'inizio di questo */
		for (new_index=0; new_index<f_ctrl->b_sector.num_block; new_index++){
			if (f_ctrl->fat_ptr[new_index] == BLOCK_FREE){
				if (file.index == BLOCK_FREE) file.index = new_index;
				else f_ctrl->fat_ptr[index] = new_index;
				f_ctrl->fat_ptr[new_index] = LAST_BLOCK;
				index = new_index;
				break;
			}
		}
		/* Controllo che la fat non sia piena */
		if (new_index>=f_ctrl->b_sector.num_block){
			fseek(fs, sizeof(struct boot_sector), SEEK_SET);
			if (!(fread(f_ctrl->fat_ptr, (f_ctrl->b_sector.num_block * sizeof(unsigned int)), 1, fs)))
				return EWFCD;
			return ENMSD;
		}
		fs_pos = f_ctrl->blk_base + (index * f_ctrl->b_sector.block_size);
	}
	fseek(fs, fs_pos, SEEK_SET);
	start_index = index;
	
	/* Conteggio i nuovi blocchi da allocare nella fat */
	dati_len = len + data_len;
	while (dati_len > f_ctrl->b_sector.block_size){
		dati_len -= f_ctrl->b_sector.block_size;
		for (new_index=0; new_index<f_ctrl->b_sector.num_block; new_index++){
			if (f_ctrl->fat_ptr[new_index] == BLOCK_FREE){
				if (file.index == BLOCK_FREE) file.index = new_index;
				else f_ctrl->fat_ptr[index] = new_index;
				f_ctrl->fat_ptr[new_index] = LAST_BLOCK;
				index = new_index;
				break;
			}
		}
		/* Controllo che la fat non sia piena */
		if (new_index>=f_ctrl->b_sector.num_block){
			fseek(fs, sizeof(struct boot_sector), SEEK_SET);
			if (!(fread(f_ctrl->fat_ptr, (f_ctrl->b_sector.num_block * sizeof(unsigned int)), 1, fs)))
				return EWFCD;
			return ENMSD;
		}	

	}

	my_start=len;
	len=data_len;
	index = start_index;
	dati = data;
	while(len){
		if ((my_start+len) > f_ctrl->b_sector.block_size){
			da_scrivere = f_ctrl->b_sector.block_size - my_start;
			len -= da_scrivere;
			my_start = 0;
		}
		else if ((my_start+len) <= f_ctrl->b_sector.block_size)
			da_scrivere = len;

		if (!(fwrite(dati, da_scrivere, 1, fs))) return ERBD;
		dati+=da_scrivere;
		if (da_scrivere == len) break;
		/* Altrimenti spostati nel blocco successivo della FAT */
		if (f_ctrl->fat_ptr[index]==BLOCK_FREE) return ERFCD;
		fs_pos = f_ctrl->blk_base + (f_ctrl->fat_ptr[index] * f_ctrl->b_sector.block_size);
		fseek(fs, fs_pos, SEEK_SET);
		index = f_ctrl->fat_ptr[index];
	}
	
	/* Aggiorno i dati della entry del file */
	fseek(fs, fs_entry, SEEK_SET);
	file.len += data_len;
	if (!(fwrite(&file, sizeof(struct dir_entry), 1, fs))) return ERBD;
	
	/* Aggiorno i dati della FAT */
	fseek(fs, sizeof(struct boot_sector), SEEK_SET);
	if (!(fwrite(f_ctrl->fat_ptr, (f_ctrl->b_sector.num_block * sizeof(unsigned int)), 1, fs)))
		return EWFCD;
	
	return 0;
}
